<?php
namespace EMS\PortalFuncionario\Geral\Services;

use Zeedhi\Framework\Controller\Crud;
use Doctrine\ORM\EntityManager;

use EMS\PortalFuncionario\Geral\Repository\AcompanhamentoRepository;
use EMS\PortalFuncionario\Geral\Repository\EscalaRepository;
use EMS\PortalFuncionario\Geral\Services\{ ApuracaoService, PeriodoApuracaoService };
use EMS\PortalFuncionario\Util\DateUtil;

use EMS\PortalFuncionario\Util\Messages;

class AcompanhamentoService extends Crud {

    /** @var EscalaRepository */
    protected $escalaRepository;
    
    public function __construct(
        EntityManager               $entityManager,
        ApuracaoService             $apuracaoService,
        PeriodoApuracaoService      $periodoApuracaoService,
        AcompanhamentoRepository    $acompanhamentoRepository,
        EscalaRepository            $escalaRepository
    ) {

        $this->entityManager            = $entityManager;
        $this->apuracaoService          = $apuracaoService;
        $this->periodoApuracaoService   = $periodoApuracaoService;
        $this->acompanhamentoRepository = $acompanhamentoRepository;
        $this->escalaRepository         = $escalaRepository;
    }

    public function getAcompanhamento(
        string  $nrorg,
        string  $nrvinculom,
        string  $nrperiodoapuracao,
        string  $dtmescompetencia           = null,
        bool    $useFinalDateAsCompetencia  = true
    ) : array {

        try {

            $periodo = $this->periodoApuracaoService->getPeriodoByNrperiodoapuracao($nrorg, $nrperiodoapuracao);
            if (empty($periodo)) {
                throw new \Exception(Messages::PER_PERIODO_NOT_FOUND);
            }
    
            $dtinicialapuracao  = $periodo['DTINICIALAPURACAO'];
            $dtfinalapuracao    = $periodo['DTFINALAPURACAO'];

            $acompanhamento = $this->acompanhamentoRepository->getAcompanhamento(
                $nrorg,
                $nrvinculom,
                $nrperiodoapuracao,
                $dtinicialapuracao,
                $dtfinalapuracao
            );

            $dtmescompetencia = $useFinalDateAsCompetencia ? $dtfinalapuracao : $dtmescompetencia;
            
            // Adiciona informações de resumo de situação de frequência
            $acompanhamento = $this->getResumoSituFreqByAcompanhamento(
                $acompanhamento,
                $nrorg,
                $nrvinculom,
                $nrperiodoapuracao,
                $dtmescompetencia
            );

            return $acompanhamento;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     *  Rotina responsável por trazer informações relativas a PONTODEMONST_DADOS para o acompanhamento
     * 
     *  @param Array [ACOMPANHAMENTO] $acompanhamento
     *  [ACOMPANHAMENTO]: 
     *      ["DATE_ORDER"]
     *      ["DTDATA"]
     *      ["DTAPURACAOSITUFREQ"]
     *      ["SALDO_A_FOLGAR_NORMAL"]
     *      ["SALDO_FOLGAR_PER_BHR_NORM"]
     *      ["SALDO_FOLGAR_ACR_PER_BHR_NORM"]
     *      ["EX_DIA_NORMAL_SEG"]
     *      ["ATRASOS_DIURNOS"]
     *      ["FALTAS_DIURNAS"]
     *      ["ISFERIADO"]
     *      ["NRVINCULOM"]
     * 
     *  @return Array [ACOMPANHAMENTORESUMOSITUFREQ] $acompanhamento
     *      [ACOMPANHAMENTORESUMOSITUFREQ]: ["DATE_ORDER"]; ["DTDATA"]; ["DTAPURACAOSITUFREQ"]; ["SALDO_A_FOLGAR_NORMAL"]; ["SALDO_FOLGAR_PER_BHR_NORM"]; ["SALDO_FOLGAR_ACR_PER_BHR_NORM"];
     *          ["EX_DIA_NORMAL_SEG"]; ["ATRASOS_DIURNOS"]; ["FALTAS_DIURNAS"]; ["ISFERIADO"]; ["NRVINCULOM"]; ["OCORRENCIA_FERIADO"]; ["OCORRENCIAS_PENDENTES"];
     *          ["FALTA_IDENTIFICADA"]; ["HORASDECONTADAS"]; ["HORASBANCODIA"]; ["SOLICITOU_COMPENSACAO_BH"]; ["HORASEXTRAS"]; ["HORASEXTRASAUTORIZADA"];
     *          ["SOLICITOU_HORAEXTRA"]; ["ID_SITUACAO_HE"]; ["FALTA_MARCACAO_ALMOCO"]; ["SOLICITOU_ABONAR_FALTA"]; ["ID_SITUACAO_ABONARFALTA"];
     *          ["IDTIPODIACALENDARIO_OCOR"]; ["NRPROGOCORRENCIA_OCOR"]; ["DSOBSERVACAO_OCOR"]; ["DSOBSERVACAORECUSA_OCOR"]; ["SOLICITOU_COMPENSAR_FALTA"];
     *          ["ID_SITUACAO_COMPENSARFALTA"]; ["ENTRADA"]; ["ALMOCO"]; ["SAIDA"]; ["INJUSTIFICADAS"]; ["JUSTIFICADAS"]; ["TOTAL_AUSENCIA"];
     *          ["HORASTRABALHADASEFETIVAS"]; ["HORASTRABALHADAS"]; ["DIFERENCAJORNADA"]; ["SALDO_BH_ANTERIOR"]; ["SALDO_DIA"]; ["BH_S_AC"]; ["SALDO_BH_ATUAL"];
     *          ["FL_ATRA"]; ["FL_DIAS"]; ["HORASNORMAIS"]
     **/ 
    public function getResumoSituFreqByAcompanhamento(
        Array   $acompanhamento,
        string  $nrorg,
        string  $nrvinculom,
        string  $nrperiodoapuracao,
        string  $dtmescompetencia
    ) {
        foreach ($acompanhamento as &$currentAcompanhamento) {

            $acompanhamentoInconsistente =
                $nrvinculom != $currentAcompanhamento['NRVINCULOM'];

            if ($acompanhamentoInconsistente) {
                continue;
            }

            $dtapuracaoponto = $currentAcompanhamento['DTDATA'];

            $resumoSituFreq = $this->apuracaoService->getApuracaoPorData(
                $nrvinculom,
                $nrorg,
                $nrperiodoapuracao,
                $dtapuracaoponto,
                $dtmescompetencia
            );

            // deixar desse jeito pra trazer os nomes dos dias da semana
            $currentAcompanhamento = array_merge($resumoSituFreq, $currentAcompanhamento);
        }

        $nrOrg     = (int) $nrorg;
        $nrVinculo = (int) $nrvinculom;

        $this->addIsDescandoProp($nrOrg, $nrVinculo, $acompanhamento);

        return $acompanhamento;
    }

    public function addIsDescandoProp(
        int $nrOrg,
        int $nrVinculo,
        array &$acompanhamento
    ): array {
        $escala = $this->escalaRepository->getEscalaTrabalhoVinculo(
            $nrOrg, $nrVinculo, true
        );

        $diasDescanso = (string) $escala['DESCANSOSEMANAL'];

        $diasSemana = [
            'DOM',
            'SEG',
            'TER',
            'QUAR',
            'QUI',
            'SEX',
            'SAB',
        ];

        foreach ($acompanhamento as &$diaAcompanhamento) {
            $date = $diaAcompanhamento['DTDATA'];
            $dateTime = DateUtil::getDataDeString($date);
            $diaSemanaNumber = DateUtil::getWeekDay($dateTime);

            $diaSemana = $diasSemana[$diaSemanaNumber];
            $isDescanso = strpos($diasDescanso, $diaSemana) !== false;

            $diaAcompanhamento['ISDESCANSO'] = $isDescanso;
        }

        return $acompanhamento;
    }

}